//
// Section: CSE 154
// Sample Dictionary code (used in 18sp midterm)
// This version fo the file uses local storage to store the definitions.
//
(function() {


  /**
   * Connect the add-entry button with the event handler
   */
  window.onload = function() {
    $("add-entry").onclick = addEntry;

    // if we were to do this right we would go through all of the terms and definitions
    // that are currently stored in the localStorage and re-load them into #current-entries
  };

  /**
   * Function to handle adding an entry to the database.
   */
  function addEntry() {
    let term = $("term").value;
    let url = "https://www.dictionaryapi.com/api/v3/references/collegiate/json/" +
              term + "?key=96b9f6b2-9412-43b8-a327-a9dc9988c4ed";
    fetch(url)
       .then(checkStatus)
       .then(JSON.parse)
       .then(processEntry)
       .catch(console.log);
  }

  function processEntry(data) {
    let definition = "";
    for (let i = 0; i < data[0].shortdef.length; i++) {
      definition += data[0].shortdef[i] + "\n";
    }
    let term = data[0].meta.id;

    if (term && definition) {

      let li = document.createElement("li");
      li.innerText = term + ": " + definition;

      // add the information to localStorage
      window.localStorage.setItem(term, definition);

      $("entries").appendChild(li);
      $("current-entries").classList.remove("hidden");

      li.ondblclick = function() {
        let term = this.innerText.split(":")[0];

        // remove item from local storage
        window.localStorage.removeItem(term, false);

        $("entries").removeChild(this);
        hideEntries();
      };
      $("term").value = "";
      $("definition").value = "";

    }
  }

  /**
   * Function to handle adding an entry to the database.
   */
  function hideEntries() {
    // no entries in current-entries
    if (!(qsa("li").length || $("current-entries").classList.contains("hidden"))) {
      $("current-entries").classList.add("hidden");
    }
  }


  // ------------------------- Helper Functions ------------------------- //
  /*
   * Helper function to return the response's result text if successful, otherwise
   * returns the rejected Promise result with an error status and corresponding text
   * @param {object} response - response to check for success/error
   * @return {object} - valid result text if response was successful, otherwise rejected
   *                    Promise result
   */
  function checkStatus(response) {
    if (response.status >= 200 && response.status < 300 || response.status == 0) {
      return response.text();
    } else {
      return Promise.reject(new Error(response.status + ": " + response.statusText));
    }
  }

  /**
    *  Shortcut to get the document element by id
    *  @param the string value of the ID of the DOM element you are getting
    *  @return the DOM element with that particular ID
    */
  function $(id) {
    return document.getElementById(id);
  }

  /**
    *  Shortcut to get the all of the document elements by query selector
    *  @param the string value of the query selector of the DOM elements you are getting
    *  @return the DOM elements with that particular query selector
    */
  function qsa(query) {
    return document.querySelectorAll(query);
  }

})();
